@@ -37,9 +37,9 @@ struct _GoaDaemon
 
   GDBusConnection *connection;
 
-  GFileMonitor *system_conf_dir_monitor;
   GFileMonitor *home_conf_file_monitor;
-  GFileMonitor *home_conf_dir_monitor;
+  GFileMonitor *template_file_monitor;
+  gchar *home_conf_file_path;
 
   GNetworkMonitor *network_monitor;
 
@@ -150,22 +150,19 @@ goa_daemon_finalize (GObject *object)
       g_source_remove (self->credentials_timeout_id);
     }
 
-  if (self->system_conf_dir_monitor != NULL)
-    {
-      g_signal_handlers_disconnect_by_func (self->system_conf_dir_monitor, on_file_monitor_changed, self);
-      g_object_unref (self->system_conf_dir_monitor);
-    }
-  if (self->home_conf_dir_monitor != NULL)
-    {
-      g_signal_handlers_disconnect_by_func (self->home_conf_dir_monitor, on_file_monitor_changed, self);
-      g_object_unref (self->home_conf_dir_monitor);
-    }
   if (self->home_conf_file_monitor != NULL)
     {
       g_signal_handlers_disconnect_by_func (self->home_conf_file_monitor, on_file_monitor_changed, self);
       g_object_unref (self->home_conf_file_monitor);
     }
 
+  if (self->template_file_monitor != NULL)
+    {
+      g_signal_handlers_disconnect_by_func (self->template_file_monitor, on_file_monitor_changed, self);
+      g_object_unref (self->template_file_monitor);
+    }
+
+  g_free (self->home_conf_file_path);
   g_object_unref (self->manager);
   g_object_unref (self->object_manager);
   g_object_unref (self->connection);
@@ -306,8 +303,10 @@ goa_daemon_init (GoaDaemon *self)
   g_dbus_object_manager_server_export (self->object_manager, G_DBUS_OBJECT_SKELETON (object));
   g_object_unref (object);
 
+  self->home_conf_file_path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
+
   /* create ~/.config/goa-1.0 directory */
-  path = g_strdup_printf ("%s/goa-1.0", g_get_user_config_dir ());
+  path = g_path_get_dirname (self->home_conf_file_path);
   if (g_mkdir_with_parents (path, 0755) != 0)
     {
       g_warning ("Error creating directory %s: %s", path, strerror (errno));
@@ -315,11 +314,16 @@ goa_daemon_init (GoaDaemon *self)
   g_free (path);
 
   /* set up file monitoring */
-  path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
-  self->home_conf_file_monitor = create_monitor (path, FALSE);
+  self->home_conf_file_monitor = create_monitor (self->home_conf_file_path, FALSE);
   if (self->home_conf_file_monitor != NULL)
     g_signal_connect (self->home_conf_file_monitor, "changed", G_CALLBACK (on_file_monitor_changed), self);
-  g_free (path);
+
+  if (GOA_TEMPLATE_FILE != NULL && GOA_TEMPLATE_FILE[0] != '\0')
+    {
+      self->template_file_monitor = create_monitor (GOA_TEMPLATE_FILE, FALSE);
+      if (self->template_file_monitor != NULL)
+        g_signal_connect (self->template_file_monitor, "changed", G_CALLBACK (on_file_monitor_changed), self);
+    }
 
   /* prime the list of accounts */
   goa_daemon_reload_configuration (self);
@@ -405,19 +409,26 @@ diff_sorted_lists (GList *list1,
 /* ---------------------------------------------------------------------------------------------------- */
 
 static const gchar *
-group_to_id (const gchar *group)
+account_group_to_id (const gchar *group)
 {
   g_return_val_if_fail (g_str_has_prefix (group, "Account "), NULL);
   return group + sizeof "Account " - 1;
 }
 
 static gchar *
-object_path_to_group (const gchar *object_path)
+account_object_path_to_group (const gchar *object_path)
 {
   g_return_val_if_fail (g_str_has_prefix (object_path, "/org/gnome/OnlineAccounts/Accounts/"), NULL);
   return g_strdup_printf ("Account %s", object_path + sizeof "/org/gnome/OnlineAccounts/Accounts/" - 1);
 }
 
+static const gchar *
+template_group_to_id (const gchar *group)
+{
+  g_return_val_if_fail (g_str_has_prefix (group, "Template "), NULL);
+  return group + sizeof "Template " - 1;
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 typedef struct
@@ -468,7 +479,7 @@ add_config_file (GoaDaemon     *self,
                                   G_KEY_FILE_NONE,
                                   &error))
     {
-      if (!(error->domain == G_FILE_ERROR && error->code == G_FILE_ERROR_NOENT))
+      if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
         {
           g_warning ("Error loading %s: %s (%s, %d)",
                      path,
@@ -510,7 +521,7 @@ add_config_file (GoaDaemon     *self,
                   g_debug ("ignoring account \"%s\" in file %s because it's stale",
                            groups[n], path);
 
-                  id = group_to_id (groups[n]);
+                  id = account_group_to_id (groups[n]);
                   if (id == NULL)
                     {
                       g_warning ("Unable to get account ID from group: %s", groups[n]);
@@ -551,6 +562,12 @@ add_config_file (GoaDaemon     *self,
               needs_update = g_key_file_remove_key (key_file, groups[n], "SessionId", NULL);
             }
 
+          g_hash_table_insert (group_name_to_key_file_data,
+                               groups[n], /* steals string */
+                               key_file_data_new (key_file, path));
+        }
+      else if (g_str_has_prefix (groups[n], "Template "))
+        {
           g_hash_table_insert (group_name_to_key_file_data,
                                groups[n], /* steals string */
                                key_file_data_new (key_file, path));
@@ -702,7 +719,10 @@ process_config_entries (GoaDaemon  *self,
       const gchar *id;
       gchar *object_path;
 
-      id = group_to_id (group);
+      if (!g_str_has_prefix (group, "Account "))
+        continue;
+
+      id = account_group_to_id (group);
 
       /* create and validate object path */
       object_path = g_strdup_printf ("/org/gnome/OnlineAccounts/Accounts/%s", id);
@@ -746,7 +766,7 @@ process_config_entries (GoaDaemon  *self,
 
       g_debug ("adding %s", object_path);
 
-      group = object_path_to_group (object_path);
+      group = account_object_path_to_group (object_path);
       key_file_data = g_hash_table_lookup (group_name_to_key_file_data, group);
       g_warn_if_fail (key_file_data != NULL);
 
@@ -779,7 +799,7 @@ process_config_entries (GoaDaemon  *self,
 
       g_debug ("unchanged %s", object_path);
 
-      group = object_path_to_group (object_path);
+      group = account_object_path_to_group (object_path);
       key_file_data = g_hash_table_lookup (group_name_to_key_file_data, group);
       g_warn_if_fail (key_file_data != NULL);
 
@@ -811,6 +831,197 @@ process_config_entries (GoaDaemon  *self,
   g_list_free_full (config_object_paths, g_free);
 }
 
+/* ---------------------------------------------------------------------------------------------------- */
+
+static gint
+compare_account_and_template_groups (const gchar *account_group, const gchar *template_group)
+{
+  const gchar *account_id;
+  const gchar *template_id;
+
+  g_return_val_if_fail (g_str_has_prefix (account_group, "Account "), 0);
+  g_return_val_if_fail (g_str_has_prefix (template_group, "Template "), 0);
+
+  account_id = account_group + sizeof "Account " - 1;
+  template_id = template_group + sizeof "Template " - 1;
+
+  return g_strcmp0 (account_id, template_id);
+}
+
+static void
+process_template_entries (GoaDaemon  *self,
+                          GHashTable *group_name_to_key_file_data)
+{
+  GError *error;
+  GHashTable *key_files_to_update = NULL;
+  GHashTableIter iter;
+  GKeyFile *home_conf_key_file = NULL;
+  GKeyFile *key_file;
+  KeyFileData *key_file_data;
+  const gchar *group;
+  const gchar *key_file_path;
+  GList *config_object_groups = NULL;
+  GList *config_template_groups = NULL;
+  GList *added;
+  GList *removed;
+  GList *unchanged;
+  GList *l;
+
+  key_files_to_update = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) g_key_file_unref);
+
+  g_hash_table_iter_init (&iter, group_name_to_key_file_data);
+  while (g_hash_table_iter_next (&iter, (gpointer *) &group, (gpointer *) &key_file_data))
+    {
+      if (home_conf_key_file == NULL && g_strcmp0 (key_file_data->path, self->home_conf_file_path) == 0)
+        home_conf_key_file = g_key_file_ref (key_file_data->key_file);
+
+      if (g_str_has_prefix (group, "Account "))
+        config_object_groups = g_list_prepend (config_object_groups, g_strdup (group));
+      else if (g_str_has_prefix (group, "Template "))
+        config_template_groups = g_list_prepend (config_template_groups, g_strdup (group));
+    }
+
+  if (home_conf_key_file == NULL)
+    home_conf_key_file = g_key_file_new ();
+
+  config_object_groups = g_list_sort (config_object_groups, (GCompareFunc) g_strcmp0);
+  config_template_groups = g_list_sort (config_template_groups, (GCompareFunc) g_strcmp0);
+  diff_sorted_lists (config_object_groups,
+                     config_template_groups,
+                     (GCompareFunc) compare_account_and_template_groups,
+                     &added,
+                     &removed,
+                     &unchanged);
+
+  for (l = added; l != NULL; l = l->next)
+    {
+      gboolean needs_update;
+      const gchar *id;
+      const gchar *template_group = l->data;
+      gchar *object_group = NULL;
+
+      key_file_data = g_hash_table_lookup (group_name_to_key_file_data, template_group);
+      g_assert_nonnull (key_file_data);
+
+      if (goa_utils_keyfile_get_boolean (key_file_data->key_file, template_group, "ForceRemove"))
+        continue;
+
+      g_debug ("Adding from template %s", template_group);
+
+      id = template_group_to_id (template_group);
+      object_group = g_strdup_printf ("Account %s", id);
+      g_warn_if_fail (!g_key_file_has_group (home_conf_key_file, object_group));
+
+      needs_update = goa_utils_keyfile_copy_group (key_file_data->key_file,
+                                                   template_group,
+                                                   home_conf_key_file,
+                                                   object_group);
+
+      if (needs_update)
+        {
+          g_key_file_set_boolean (home_conf_key_file, object_group, "IsLocked", TRUE);
+          g_hash_table_insert (key_files_to_update,
+                               g_strdup (self->home_conf_file_path),
+                               g_key_file_ref (home_conf_key_file));
+        }
+
+      g_free (object_group);
+    }
+
+  for (l = unchanged; l != NULL; l = l->next)
+    {
+      KeyFileData *object_key_file_data;
+      KeyFileData *template_key_file_data;
+      gboolean needs_update;
+      const gchar *id;
+      const gchar *object_group = l->data;
+      gchar *template_group = NULL;
+
+      object_key_file_data = g_hash_table_lookup (group_name_to_key_file_data, object_group);
+      g_assert_nonnull (object_key_file_data);
+
+      g_warn_if_fail (g_key_file_has_group (object_key_file_data->key_file, object_group));
+
+      id = account_group_to_id (object_group);
+      template_group = g_strdup_printf ("Template %s", id);
+
+      template_key_file_data = g_hash_table_lookup (group_name_to_key_file_data, template_group);
+      g_assert_nonnull (template_key_file_data);
+      g_assert_true (g_key_file_has_group (template_key_file_data->key_file, template_group));
+
+      if (goa_utils_keyfile_get_boolean (template_key_file_data->key_file, template_group, "ForceRemove"))
+        {
+          gboolean removed;
+
+          g_debug ("Template %s specifies ForceRemove, removing %s", template_group, object_group);
+
+          error = NULL;
+          needs_update = g_key_file_remove_group (object_key_file_data->key_file, object_group, &error);
+          if (error != NULL)
+            {
+              g_warning ("Error removing group %s from %s: %s (%s, %d)",
+                         object_group,
+                         key_file_data->path,
+                         error->message,
+                         g_quark_to_string (error->domain),
+                         error->code);
+              g_error_free (error);
+            }
+
+          if (needs_update)
+            {
+              g_hash_table_insert (key_files_to_update,
+                                   g_strdup (object_key_file_data->path),
+                                   g_key_file_ref (object_key_file_data->key_file));
+            }
+
+          removed = g_hash_table_remove (group_name_to_key_file_data, object_group);
+          g_warn_if_fail (removed);
+        }
+      else
+        {
+          g_debug ("Updating %s from template %s", object_group, template_group);
+
+          needs_update = goa_utils_keyfile_copy_group (template_key_file_data->key_file,
+                                                       template_group,
+                                                       object_key_file_data->key_file,
+                                                       object_group);
+
+          if (needs_update)
+            {
+              g_key_file_set_boolean (home_conf_key_file, object_group, "IsLocked", TRUE);
+              g_hash_table_insert (key_files_to_update,
+                                   g_strdup (object_key_file_data->path),
+                                   g_key_file_ref (object_key_file_data->key_file));
+            }
+        }
+
+      g_free (template_group);
+    }
+
+  g_hash_table_iter_init (&iter, key_files_to_update);
+  while (g_hash_table_iter_next (&iter, (gpointer *) &key_file_path, (gpointer *) &key_file))
+    {
+      error = NULL;
+      if (!g_key_file_save_to_file (key_file, key_file_path, &error))
+        {
+          g_prefix_error (&error, "Error writing key-value-file %s: ", key_file_path);
+          g_warning ("%s (%s, %d)", error->message, g_quark_to_string (error->domain), error->code);
+          g_error_free (error);
+        }
+    }
+
+  g_hash_table_unref (key_files_to_update);
+  g_key_file_unref (home_conf_key_file);
+  g_list_free (removed);
+  g_list_free (added);
+  g_list_free (unchanged);
+  g_list_free_full (config_object_groups, g_free);
+  g_list_free_full (config_template_groups, g_free);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 /* <internal>
  * goa_daemon_reload_configuration:
  * @self: A #GoaDaemon
@@ -823,7 +1034,6 @@ static void
 goa_daemon_reload_configuration (GoaDaemon *self)
 {
   GHashTable *group_name_to_key_file_data;
-  gchar *path;
 
   group_name_to_key_file_data = g_hash_table_new_full (g_str_hash,
                                                        g_str_equal,
@@ -831,9 +1041,12 @@ goa_daemon_reload_configuration (GoaDaemon *self)
                                                        (GDestroyNotify) key_file_data_free);
 
   /* Read the main user config file at $HOME/.config/goa-1.0/accounts.conf */
-  path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
-  add_config_file (self, path, group_name_to_key_file_data);
-  g_free (path);
+  add_config_file (self, self->home_conf_file_path, group_name_to_key_file_data);
+
+  if (GOA_TEMPLATE_FILE != NULL && GOA_TEMPLATE_FILE[0] != '\0')
+    add_config_file (self, GOA_TEMPLATE_FILE, group_name_to_key_file_data);
+
+  process_template_entries (self, group_name_to_key_file_data);
 
   /* now process the group_name_to_key_file_data hash table */
   process_config_entries (self, group_name_to_key_file_data);
@@ -883,7 +1096,6 @@ get_all_providers_cb (GObject      *source,
   GError *error;
   GList *providers = NULL;
   GList *l;
-  gchar *path = NULL;
   gchar *id = NULL;
   gchar *group = NULL;
   gchar *key_file_data = NULL;
@@ -926,9 +1138,9 @@ get_all_providers_cb (GObject      *source,
     }
 
   key_file = g_key_file_new ();
-  path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
+
   error = NULL;
-  if (!g_file_get_contents (path,
+  if (!g_file_get_contents (data->daemon->home_conf_file_path,
                             &key_file_data,
                             &length,
                             &error))
@@ -939,7 +1151,7 @@ get_all_providers_cb (GObject      *source,
         }
       else
         {
-          g_prefix_error (&error, "Error loading file %s: ", path);
+          g_prefix_error (&error, "Error loading file %s: ", data->daemon->home_conf_file_path);
           g_dbus_method_invocation_take_error (data->invocation, error);
           goto out;
         }
@@ -951,7 +1163,7 @@ get_all_providers_cb (GObject      *source,
           error = NULL;
           if (!g_key_file_load_from_data (key_file, key_file_data, length, G_KEY_FILE_KEEP_COMMENTS, &error))
             {
-              g_prefix_error (&error, "Error parsing key-value-file %s: ", path);
+              g_prefix_error (&error, "Error parsing key-value-file %s: ", data->daemon->home_conf_file_path);
               g_dbus_method_invocation_take_error (data->invocation, error);
               goto out;
             }
@@ -992,9 +1204,9 @@ get_all_providers_cb (GObject      *source,
     }
 
   error = NULL;
-  if (!g_key_file_save_to_file (key_file, path, &error))
+  if (!g_key_file_save_to_file (key_file, data->daemon->home_conf_file_path, &error))
     {
-      g_prefix_error (&error, "Error writing key-value-file %s: ", path);
+      g_prefix_error (&error, "Error writing key-value-file %s: ", data->daemon->home_conf_file_path);
       g_dbus_method_invocation_take_error (data->invocation, error);
       goto out;
     }
@@ -1023,7 +1235,6 @@ get_all_providers_cb (GObject      *source,
   g_free (key_file_data);
   g_free (group);
   g_free (id);
-  g_free (path);
   g_clear_pointer (&key_file, (GDestroyNotify) g_key_file_unref);
   g_object_unref (data->daemon);
   g_object_unref (data->manager);
@@ -1138,7 +1349,6 @@ on_account_handle_remove (GoaAccount            *account,
   GTask *task = NULL;
   ObjectInvocationData *data;
   const gchar *provider_type = NULL;
-  gchar *path = NULL;
   gchar *group = NULL;
   GError *error;
 
@@ -1158,11 +1368,10 @@ on_account_handle_remove (GoaAccount            *account,
    */
 
   key_file = g_key_file_new ();
-  path = g_strdup_printf ("%s/goa-1.0/accounts.conf", g_get_user_config_dir ());
 
   error = NULL;
   if (!g_key_file_load_from_file (key_file,
-                                  path,
+                                  self->home_conf_file_path,
                                   G_KEY_FILE_KEEP_COMMENTS,
                                   &error))
     {
@@ -1180,9 +1389,9 @@ on_account_handle_remove (GoaAccount            *account,
     }
 
   error = NULL;
-  if (!g_key_file_save_to_file (key_file, path, &error))
+  if (!g_key_file_save_to_file (key_file, self->home_conf_file_path, &error))
     {
-      g_prefix_error (&error, "Error writing key-value-file %s: ", path);
+      g_prefix_error (&error, "Error writing key-value-file %s: ", self->home_conf_file_path);
       g_dbus_method_invocation_take_error (invocation, error);
       goto out;
     }
@@ -1232,7 +1441,6 @@ on_account_handle_remove (GoaAccount            *account,
   g_clear_object (&task);
   g_clear_pointer (&key_file, (GDestroyNotify) g_key_file_unref);
   g_free (group);
-  g_free (path);
   return TRUE; /* invocation was handled */
 }
 
