@@ -520,6 +520,7 @@ create_account_details_ui (GoaProvider    *provider,
                            GtkDialog      *dialog,
                            GtkBox         *vbox,
                            gboolean        new_account,
+                           gboolean        is_template,
                            AddAccountData *data)
 {
   GtkWidget *grid0;
@@ -560,7 +561,12 @@ create_account_details_ui (GoaProvider    *provider,
   add_entry (grid1, row++, _("_Password"), &data->password);
   gtk_entry_set_visibility (GTK_ENTRY (data->password), FALSE);
 
-  gtk_widget_grab_focus ((new_account) ? data->uri : data->password);
+  if (new_account)
+    gtk_widget_grab_focus (data->uri);
+  else if (is_template)
+    gtk_widget_grab_focus (data->username);
+  else
+    gtk_widget_grab_focus (data->password);
 
   g_signal_connect (data->uri, "changed", G_CALLBACK (on_uri_username_or_password_changed), data);
   g_signal_connect (data->username, "changed", G_CALLBACK (on_uri_username_or_password_changed), data);
@@ -680,7 +686,7 @@ add_account (GoaProvider    *provider,
   data.dialog = dialog;
   data.error = NULL;
 
-  create_account_details_ui (provider, dialog, vbox, TRUE, &data);
+  create_account_details_ui (provider, dialog, vbox, TRUE, FALSE, &data);
   gtk_widget_show_all (GTK_WIDGET (vbox));
   g_signal_connect (dialog, "response", G_CALLBACK (dialog_response_cb), &data);
 
@@ -837,12 +843,13 @@ refresh_account (GoaProvider    *provider,
                  GError        **error)
 {
   AddAccountData data;
-  GVariantBuilder builder;
+  GVariantBuilder credentials;
   GoaAccount *account;
   GoaHttpClient *http_client;
   GtkWidget *dialog;
   GtkWidget *vbox;
   gboolean accept_ssl_errors;
+  gboolean is_template;
   gboolean ret;
   const gchar *password;
   const gchar *username;
@@ -857,6 +864,7 @@ refresh_account (GoaProvider    *provider,
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   http_client = NULL;
+  is_template = FALSE;
   uri = NULL;
   uri_webdav = NULL;
 
@@ -882,17 +890,23 @@ refresh_account (GoaProvider    *provider,
   data.dialog = GTK_DIALOG (dialog);
   data.error = NULL;
 
-  create_account_details_ui (provider, GTK_DIALOG (dialog), GTK_BOX (vbox), FALSE, &data);
+  account = goa_object_peek_account (object);
+  username = goa_account_get_identity (account);
+  if (username == NULL || username[0] == '\0')
+    is_template = TRUE;
+
+  create_account_details_ui (provider, GTK_DIALOG (dialog), GTK_BOX (vbox), FALSE, is_template, &data);
 
   accept_ssl_errors = goa_util_lookup_keyfile_boolean (object, "AcceptSslErrors");
   uri = goa_util_lookup_keyfile_string (object, "Uri");
   gtk_entry_set_text (GTK_ENTRY (data.uri), uri);
   gtk_editable_set_editable (GTK_EDITABLE (data.uri), FALSE);
 
-  account = goa_object_peek_account (object);
-  username = goa_account_get_identity (account);
-  gtk_entry_set_text (GTK_ENTRY (data.username), username);
-  gtk_editable_set_editable (GTK_EDITABLE (data.username), FALSE);
+  if (!is_template)
+    {
+      gtk_entry_set_text (GTK_ENTRY (data.username), username);
+      gtk_editable_set_editable (GTK_EDITABLE (data.username), FALSE);
+    }
 
   gtk_widget_show_all (dialog);
   g_signal_connect (dialog, "response", G_CALLBACK (dialog_response_cb), &data);
@@ -911,6 +925,9 @@ refresh_account (GoaProvider    *provider,
       goto out;
     }
 
+  if (is_template)
+    username = gtk_entry_get_text (GTK_ENTRY (data.username));
+
   password = gtk_entry_get_text (GTK_ENTRY (data.password));
   g_cancellable_reset (data.cancellable);
   goa_http_client_check (http_client,
@@ -954,15 +971,55 @@ refresh_account (GoaProvider    *provider,
     }
 
   /* TODO: run in worker thread */
-  g_variant_builder_init (&builder, G_VARIANT_TYPE_VARDICT);
-  g_variant_builder_add (&builder, "{sv}", "password", g_variant_new_string (password));
+  g_variant_builder_init (&credentials, G_VARIANT_TYPE_VARDICT);
+  g_variant_builder_add (&credentials, "{sv}", "password", g_variant_new_string (password));
 
-  if (!goa_utils_store_credentials_for_object_sync (provider,
-                                                    object,
-                                                    g_variant_builder_end (&builder),
-                                                    NULL, /* GCancellable */
-                                                    &data.error))
-    goto out;
+  if (is_template)
+    {
+      GVariantBuilder details;
+      GoaManager *manager;
+      const gchar *id;
+      const gchar *provider_type;
+      gchar *dummy = NULL;
+      gchar *presentation_identity = NULL;
+      gchar *server = NULL;
+
+      manager = goa_client_get_manager (client);
+      id = goa_account_get_id (account);
+      provider_type = goa_provider_get_provider_type (provider);
+
+      dummy = normalize_uri (uri, &server);
+      presentation_identity = g_strconcat (username, "@", server, NULL);
+      g_free (dummy);
+      g_free (server);
+
+      g_variant_builder_init (&details, G_VARIANT_TYPE ("a{ss}"));
+      g_variant_builder_add (&details, "{ss}", "Id", id);
+
+      goa_manager_call_add_account (manager,
+                                    provider_type,
+                                    username,
+                                    presentation_identity,
+                                    g_variant_builder_end (&credentials),
+                                    g_variant_builder_end (&details),
+                                    NULL, /* GCancellable* */
+                                    (GAsyncReadyCallback) add_account_cb,
+                                    &data);
+      g_free (presentation_identity);
+
+      g_main_loop_run (data.loop);
+      if (data.error != NULL)
+        goto out;
+    }
+  else
+    {
+      if (!goa_utils_store_credentials_for_object_sync (provider,
+                                                        object,
+                                                        g_variant_builder_end (&credentials),
+                                                        NULL, /* GCancellable */
+                                                        &data.error))
+        goto out;
+    }
 
   goa_account_call_ensure_credentials (account,
                                        NULL, /* GCancellable */
@@ -977,6 +1034,7 @@ refresh_account (GoaProvider    *provider,
   gtk_widget_destroy (dialog);
   g_free (uri);
   g_free (uri_webdav);
+  g_free (data.account_object_path);
   g_clear_pointer (&data.loop, (GDestroyNotify) g_main_loop_unref);
   g_clear_object (&data.cancellable);
   g_clear_object (&http_client);
